local eventCache = {}
local eventList = {
	[40200] = {
		subevent = 2,
		srcname = 5,
		dstname = 9,
		spid = 12,
		spname = 13,
		swingAmount = 12,
		spAmount = 15,
		swingAbsorbAmt = 17,
		spAbsorbAmt = 20,
		swingCritDmg = 20,
		spCriticalDmg = 21,
	},
}
local function clEvent(ver,event)
	if eventCache[ver] and eventCache[ver][event] then return eventCache[ver][event] end
	local highestEventLevel = 0
	for verNum, data in pairs(eventList) do
		if data[event] and verNum > highestEventLevel then
			highestEventLevel = verNum
		end
	end
	eventCache[ver] = eventCache[ver] or {}
	eventCache[ver][event] = eventList[highestEventLevel][event]
	return eventCache[ver][event]
end
local currentBuild = select(4,GetBuildInfo())
do
	local a = LibStub("LibSharedMedia-3.0")
	if a then
		a:Register("font", "DueLog", "Interface\\AddOns\\DueLog\\font.ttf", a.LOCALE_BIT_western)
	end
end
local lc = LibStub("LibCompress")
local as = LibStub("AceSerializer-3.0")
DueLog = {}
DueLog.dialogsused = {false,false,false,false}
for i=1, 4 do
	StaticPopupDialogs["DUELOG_TRANSMIT"..i] = {
		button1=ACCEPT,
		button2=CANCEL,
		timeout = 0,
		whileDead = 1,
		hideOnEscape = 0,
	}
end
DueLog.frame = CreateFrame("Frame")
local rp = print
local function print(alwaysprint, ...)
	local debug = false
	--[===[@debug@
	debug=true
	--@end-debug@]===]
	if alwaysprint or debug then rp(...) end
end
print(true, "DueLog loaded")
local function SendAddonMessage(prefix, msg, channel, dest, alert, queue, cb, cba)
	ChatThrottleLib:SendAddonMessage(alert and "ALERT" or "BULK", prefix, msg, channel, dest, queue, cb, cba)
end
local function round(num, places)
	return tonumber(string.format("%."..(places or 0).."f",num))
end
local function rndhex()
	local a=""
	for i=1, 6 do
		local b = math.random(0,15)
		b = (b==15 and "f" or b==14 and "e" or b==13 and "d" or b==12 and "c" or b==11 and "b" or b==10 and "a" or tostring(b))
		a=a..b
	end
	return a
end
local function ClogSent(t)
	if not DueLog.connections then return end
	local cd = DueLog.connections[t.cid]
	cd.clog[t.num] = true
	local c = 0
	for i,k in pairs(cd.clog) do if k then c = c+1 end end
	cd.pct = (c/cd.num)
	if c == cd.num then
		DueLog.connections[t.cid] = nil print(false, "Transmission "..t.cid.." finished")
	end
end
local function Random8DigitNum()
	local a = ""
	for i=1, 8 do
		a=a..math.random(0,9)
	end
	return a
end
local function InitEventFrame()
	local f = DueLog.frame
	f.events = {}
	f:SetScript("OnEvent", function(self, event, ...)
		if not self.events[event] then return end
		for key, func in ipairs(self.events[event]) do
			func(...)
		end
	end)
end
InitEventFrame()
function DueLog:RegisterEvent(event, func)
	local f = DueLog.frame
	local e = f.events[event]
	if not e then
		f.events[event] = {}
		e=f.events[event]
		f:RegisterEvent(event)
	elseif e[tostring(func)] then return end
	tinsert(e, func)
	e[tostring(func)] = #e
end
function DueLog:UnregisterEvent(event, func)
	local f = DueLog.frame
	if not f.events[event] then return end
	local e = f.events[event]
	if not e[tostring(func)] then return end
	tremove(e,e[tostring(func)])
	e[tostring(func)] = nil
	if #e<1 then f.events[event] = nil f:UnregisterEvent(event) end
end
local function CombatLogHandler(...)
	local t = DueLog.curduel
	if not (t and t.started) then return end
	if not t.combatlog then t.combatlog = {} end
	local x = {...}
	x.time = GetTime()
	tinsert(t.combatlog, x)
end
local function SpellCastSucceededHandler(unitid, spell)
	local t = DueLog.curduel
	if not (t and t.started) then return end
	if not t.combatlog then t.combatlog = {} end
	tinsert(t.combatlog, {[clEvent(currentBuild,"subevent")]="UNIT_SPELLCAST_SUCCEEDED", [clEvent(currentBuild,"srcname")]=UnitName(unitid), [clEvent(currentBuild,"spname")]=spell, time=GetTime()})
end
function DueLog:ShowPopup(popuptype, arg)
	if popuptype == "inc_transmit" then
		local popupid
		for k=1, 4 do
			if not DueLog.dialogsused[k] then
				DueLog.dialogsused[k] = true
				i = k
				break
			end
		end
		if i then
			local c = StaticPopupDialogs["DUELOG_TRANSMIT"..i]
			local d = DueLog.connections[arg]
			c.text = d.t.." is trying to send you a duel:\n|cffffd400"..d.duel.duel1.." - "..d.duel.duel2.."|r?"
			c.OnAccept = function()
				DueLog.dialogsused[i] = false
				SendAddonMessage("DueLog", "ack|"..arg.."|", "WHISPER", d.t, true)
				if not InCombatLockdown() then
					DueLog:TransmitWindow()
					DueLog.transmit.window.content:SetContentTab(2)
				end
				d.pct = 0
			end
			c.OnCancel = function()
				DueLog.dialogsused[i] = false
				SendAddonMessage("DueLog", "decline|"..arg.."|", "WHISPER", d.t, true)
				DueLog.connections[arg] = nil
			end
			StaticPopup_Show("DUELOG_TRANSMIT"..i)
		else
			SendAddonMessage("DueLog", "decline|"..arg.."|", "WHISPER", DueLog.connections[arg].t, true)
			DueLog.connections[arg] = nil
		end
	end
end

function DueLog:GetAvailableConnectionID()
	if not DueLog.connections then DueLog.connections = {} end
	local a = Random8DigitNum()
	while DueLog.connections[a] do
		a=Random8DigitNum()
	end
	return a
end
function DueLog:QueueTransmission(connid)
	local cd = DueLog.connections[connid]
	if cd.compress then self:QueueCompressedTransmission(connid) end
	local d = DueLog.duels[cd.n]
	print(false, "Queueing up transmissions for connid "..connid..", duel "..cd.n.." ("..d.duel1.." - "..d.duel2..")")
	for num, data in ipairs(d.combatlog) do
		local a = {}
		a.tar = cd.t
		a.form = "WHISPER"
		a.prefix = "DueLog"
		a.msg = "clog|"..connid.."|"..num.."|"..data.time.."|"
		if data[2] ~= "UNIT_SPELLCAST_SUCCEEDED" then
			for key, k in pairs(data) do
				if tonumber(key) then
					a.msg = a.msg..key.."="..k.."|"
				end
			end
		else
			a.msg = a.msg.."UNIT_SPELLCAST_SUCCEEDED|"..data[4].."|"..data[10].."|"
		end
		SendAddonMessage(a.prefix, a.msg, a.form, a.tar, false, "DueLog"..connid, ClogSent, {cid=connid, num=num})
	end
end
function DueLog:QueueCompressedTransmission(connid)
	local cd = DueLog.connections[connid]
	local d = DueLog.duels[cd.n]
	print(false, "Queueing up compressed transmissions for connid "..connid..", duel "..cd.n.." ("..d.duel1.." - "..d.duel2..")")
	d = lc:GetAddonEncodeTable():Encode(lc:Compress(as:Serialize(d)))
	cd.num = math.ceil(strlen(d)/220)
	SendAddonMessage("DueLog", "compress_length|"..connid.."|"..math.ceil(strlen(d)/220).."|", "WHISPER", cd.t, false, "DueLog"..connid)
	local c = 1
	while strlen(d) > 220 do
		DueLog.connections[connid].clog[c] = false
		SendAddonMessage("DueLog", "clog|"..connid.."|"..c.."|"..d:sub(1,220).."|", "WHISPER", cd.t, false, "DueLog"..connid, ClogSent, {cid=connid, num=c})
		c=c+1
		d = d:sub(221)
	end
	SendAddonMessage("DueLog", "clog|"..connid.."|"..c.."|"..d.."|", "WHISPER", cd.t, false, "DueLog"..connid, ClogSent, {cid=connid, num=c})
end
	
DueLog:RegisterEvent("UNIT_SPELLCAST_SENT", function(caster, spell, rank, target)
	if not (spell==(GetSpellInfo(7266))) then return end
	print(false, "Duel challenge: "..UnitName("player").." -> "..target.."!")
	if not DueLog.curduel then DueLog.curduel = {} end
	local t = DueLog.curduel
	t.duel1 = UnitName("player")
	t.duel2 = target
end)
DueLog:RegisterEvent("DUEL_REQUESTED", function(challenger)
	print(false, "Duel challenge: "..challenger.." -> "..UnitName("player").."!")
	if not DueLog.curduel then DueLog.curduel = {} end
	local t = DueLog.curduel
	t.duel1 = challenger
	t.duel2 = UnitName("player")
end)
DueLog:RegisterEvent("CHAT_MSG_SYSTEM", function(msg)
	local t = DueLog.curduel
	if not t then return end
	if msg == "Duel starting: 3" then
		if (not t.duel1) or (not t.duel2) then print(false, t.duel1, t.duel2) return end
		print(false, "Duel is starting between "..t.duel1.." and "..t.duel2.."!")
		t.started = GetTime()+3
		DueLog:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", CombatLogHandler)
		DueLog:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", SpellCastSucceededHandler)
	elseif msg:find("%S+ has defeated %S+ in a duel") then
		local winner, loser = string.match(msg, "(%S+) has defeated (%S+) in a duel")
		if not ((winner == t.duel1 and loser == t.duel2) or (winner == t.duel2 and loser == t.duel1)) then return end
		print(false, "Duel finished. "..winner.." beat "..loser.."!")
		t.winner = winner
		t.loser = loser
		t.ended = GetTime()
		t.duration = GetTime()-t.started
		t.version = currentBuild
		if not DueLog.duels then DueLog.duels = {} end
		print(true, "The duel between "..t.duel1.." and "..t.duel2.." had started and is being saved. ("..(#DueLog.duels+1)..")")
		tinsert(DueLog.duels, DueLog.curduel)
		DueLog:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", CombatLogHandler)
		DueLog:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", SpellCastSucceededHandler)
		DueLog.curduel = {}
	elseif msg:find("%S+ has fled from %S+ in a duel") then
		local loser, winner = string.match(msg, "(%S+) has fled from (%S+) in a duel")
		if not ((winner == t.duel1 and loser == t.duel2) or (winner == t.duel2 and loser == t.duel1)) then return end
		print(false, "Duel finished. "..winner.." beat "..loser.."!")
		t.winner = winner
		t.loser = loser
		t.ended = GetTime()
		t.duration = GetTime()-t.started
		t.version = currentBuild
		if not DueLog.duels then DueLog.duels = {} end
		print(true, "The duel between "..t.duel1.." and "..t.duel2.." had started and is being saved. ("..(#DueLog.duels+1)..")")
		tinsert(DueLog.duels, DueLog.curduel)
		DueLog:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", CombatLogHandler)
		DueLog:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", SpellCastSucceededHandler)
		DueLog.curduel = {}
	end
end)
DueLog:RegisterEvent("DUEL_FINISHED", function()
	local t = DueLog.curduel
	if not t then return end
	if not (t.duel1 and t.duel2) then return end
	if not t.started then
		print(false, "The duel between "..t.duel1.." and "..t.duel2.." was not accepted and is being discarded.")
		for key in pairs(t) do
			t[key] = nil
		end
	end
end)
DueLog:RegisterEvent("UNIT_SPELLCAST_FAILED", function(unit, spell)
	if not (unit == "player" and spell == GetSpellInfo(7266)) then return end
	print(false, "The duel challenge failed and is being discarded")
	if not DueLog.curduel then return end
	for key in pairs(DueLog.curduel) do
		DueLog.curduel[key] = nil
	end
end)
DueLog:RegisterEvent("CHAT_MSG_ADDON", function(prefix, msg, chan, sender)
	if chan ~= "WHISPER" then return end
	if not prefix == "DueLog" then return end
	local key = string.match(msg, "([^|]+)|.+")
	if key == "request" then
		local duel1, duel2, duration, winner, loser, started, ended, numlog, connid, version = string.match(msg, "request|([^|]+)|([^|]+)|([^|]+)|([^|]+)|([^|]+)|([^|]+)|([^|]+)|([^|]+)|([^|]+)|([^|]+)")
		local isCompress = string.match(msg, "request|[^|]+|[^|]+|[^|]+|[^|]+|[^|]+|[^|]+|[^|]+|[^|]+|[^|]+|[^|]+|(1)")
		print(false, "Incoming"..(isCompress and " compressed" or "").." duel transmission from "..sender..": "..duel1.." - "..duel2.." (Winner: "..winner..") Duration: "..round(duration, 2).." sec (CID "..connid..")")
		if not DueLog.connections then DueLog.connections = {} end
		if DueLog.connections[connid] then
			SendAddonMessage("DueLog", "id_taken|"..connid.."|", "WHISPER", sender, true)
		else
			DueLog.dialogdata = connid
			DueLog.connections[connid] = {t=sender, role="RCVR", duel = {duel1=duel1, duel2=duel2, duration=tonumber(duration), winner=winner, loser=loser, started=tonumber(started), ended=tonumber(ended), combatlog={}, version=tonumber(version)}, num=tonumber(numlog), pct="Waiting", compress=(not not isCompress)}
			if isCompress then
				DueLog.connections[connid].cryptolog = {}
			end
			print(false, "Connection ID negotiations complete ("..connid..")")
			DueLog:ShowPopup("inc_transmit", connid)
		end
	elseif key == "id_taken" then
		local id = string.match(msg, "id_taken|(%d+)|")
		local c = DueLog:GetAvailableConnectionID()
		DueLog.connections[c] = DueLog.connections[id]
		DueLog.connections[id] = nil
		local d = DueLog.duels[DueLog.connections[c].n]
		SendAddonMessage("DueLog", "request|"..d.duel1.."|"..d.duel2.."|"..d.duration.."|"..d.winner.."|"..d.loser.."|"..d.started.."|"..d.ended.."|"..#d.combatlog.."|"..c.."|"..(d.version or 40200).."|"..(DueLog.connections[c].compress and "1|" or ""), "WHISPER", sender, true)
	elseif key == "ack" then
		local connid = string.match(msg, "ack|(%d+)|")
		print(false, "Connection ID negotiations complete ("..connid..")")
		DueLog:TransmitWindow()
		DueLog.transmit.window.content:SetContentTab(3)
		DueLog.connections[connid].pct = 0
		DueLog:QueueTransmission(connid)
	elseif key == "decline" then
		local connid = string.match(msg, "decline|(%d+)|")
		print(true, "Connection refused on connid "..connid)
		DueLog.connections[connid] = nil
	elseif key == "compress_length" then
		if not DueLog.connections then return end
		local connid, num = string.match(msg, "compress_length|(%d+)|(%d+)|")
		num = tonumber(num)
		if not DueLog.connections[connid] then return end
		DueLog.connections[connid].clength = num
	elseif key == "clog" then
		if not DueLog.connections then return end
		local connid = string.match(msg, "clog|(%d+)|")
		local cd = DueLog.connections[connid]
		if not cd then return end
		if cd.compress then
			local connid, num, datastream = string.match(msg, "clog|(%d+)|(%d+)|(.+)")
			cd.cryptolog[tonumber(num)] = datastream
			local a = cd.clength
			local b = 0
			for d in pairs(cd.cryptolog) do
				b = b+1
			end
			--SendAddonMessage("DueLog", "clog_confirm|"..connid.."|"..num.."|", "WHISPER", sender)
			cd.pct = b/a
			if b == a then
				_, cd.duel.combatlog = as:Deserialize(lc:Decompress(lc:GetAddonEncodeTable():Decode(table.concat(cd.cryptolog))))
				if not DueLog.duels then DueLog.duels = {} end
				tinsert(DueLog.duels, cd.duel)
				print(false, "Compressed transmission finished ("..#DueLog.duels..")")
				DueLog.connections[connid] = nil
			end
		else
			local version = cd.duel.version
			local connid, num, time, data = string.match(msg, "clog|(%d+)|(%d+)|([^|]+)|(.+)")
			num, time = tonumber(num), tonumber(time)
			local clg = cd.duel.combatlog
			clg[num] = {}
			if not cd then return end
			if cd.t ~= sender then return end
			if string.match(data, "[^|]+") == "UNIT_SPELLCAST_SUCCEEDED" then
				local source, spell = string.match(data, "UNIT_SPELLCAST_SUCCEEDED|([^|]+)|([^|]+)|")
				clg[num][2] = "UNIT_SPELLCAST_SUCCEEDED"
				clg[num][clEvent(version,"srcname")] = source
				clg[num][clEvent(version,"spname")] = spell
			else
				for d in string.gmatch(data, "([^|]+)|") do
					local n2, dt = string.match(d, "([^=]+)=([^|]+)")
					if string.match(dt, "[%.%d]+") == dt then
						dt = tonumber(dt)
					end
					clg[num][tonumber(n2)] = dt
				end
			end
			clg[num].time = time
			--SendAddonMessage("DueLog", "clog_confirm|"..connid.."|"..num.."|", "WHISPER", sender)
			cd.pct = #clg/cd.num
			if #clg == cd.num then
				if not DueLog.duels then DueLog.duels = {} end
				tinsert(DueLog.duels, cd.duel)
				print(false, "Transmission finished ("..#DueLog.duels..")")
				DueLog.connections[connid] = nil
			end
		end
	--[[elseif key == "clog_confirm" then
		if not DueLog.connections then return end
		local connid, num = string.match(msg, "clog_confirm|(%d+)|(%d+)|")
		DueLog.connections[connid].clog[tonumber(num)] = true
		local count = 0
		for i,k in pairs(DueLog.connections[connid].clog) do if k then count = count+1 end end
		DueLog.connections[connid].pct = (count/#DueLog.duels[DueLog.connections[connid].n].combatlog)
		if #DueLog.duels[DueLog.connections[connid].n].combatlog == count then DueLog.connections[connid] = nil print(false, "Transmission "..connid.." finished") end--]]
	end
end)

function DueLog:ListWindow()
	if not self.list then self.list = {} end
	if not self.list.window then
		self.list.window = CreateFrame("Frame", nil, UIParent)
		local w = self.list.window
		w:SetPoint("CENTER")
		w:SetWidth(600)
		w:SetHeight(500)
		w:SetFrameStrata("DIALOG")
		w.texture = w:CreateTexture(nil, "BACKGROUND")
		w.texture:SetAllPoints()
		w.texture:SetTexture(0.3,0.3,0.3,0.5)
		w:SetMovable(true)
		w.titlebar = CreateFrame("Frame", nil, w)
		local titlebar = w.titlebar
		titlebar:SetPoint("BOTTOMLEFT", w, "TOPLEFT")
		titlebar:SetPoint("BOTTOMRIGHT", w, "TOPRIGHT")
		titlebar:SetHeight(20)
		titlebar.texture = titlebar:CreateTexture(nil, "BACKGROUND")
		titlebar.texture:SetTexture(0,0,0.5,1)
		titlebar.texture:SetAllPoints()
		titlebar.text = titlebar:CreateFontString(nil)
		titlebar.text:SetPoint("TOPLEFT", 1, 0)
		titlebar.text:SetPoint("BOTTOMRIGHT", 0, 0)
		titlebar.text:SetJustifyH("LEFT")
		titlebar.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", "12")
		titlebar.text:SetTextColor(0.9,0.9,0.9)
		titlebar.text:SetText("Duel listing window")
		titlebar.closebutton = CreateFrame("Button", nil, titlebar)
		titlebar:EnableMouse(true)
		titlebar:SetScript("OnMouseDown", function() w:StartMoving() end)
		titlebar:SetScript("OnMouseUp", function() w:StopMovingOrSizing() end)
		local closebutton = titlebar.closebutton
		closebutton:RegisterForClicks("AnyUp")
		closebutton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		closebutton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
		closebutton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
		closebutton:SetScript("OnClick", function() w:Hide() end)
		closebutton:SetPoint("TOPRIGHT", 3, 3)
		closebutton:SetHeight(26)
		closebutton:SetWidth(26)
		closebutton:SetHitRectInsets(6,6,6,6)
		w.menu = {}
		w.menu[1] = CreateFrame("Frame", nil, w)
		w.menu[1]:SetPoint("TOPLEFT")
		w.menu[2] = CreateFrame("Frame", nil, w)
		w.menu[2]:SetPoint("TOPRIGHT")
		for mid=1,2 do
			w.menu[mid]:SetWidth(292.5)
			w.menu[mid]:SetHeight(20)
			w.menu[mid].texture = w.menu[mid]:CreateTexture(nil, "BACKGROUND")
			w.menu[mid].texture:SetTexture(0.5,0.5,0.5,0.5)
			w.menu[mid].texture:SetAllPoints()
			w.menu[mid].text = w.menu[mid]:CreateFontString(nil, "ARTWORK")
			w.menu[mid].text:SetAllPoints()
			w.menu[mid].text:SetFont("Interface\\Addons\\DueLog\\font.ttf", "12")
			w.menu[mid].text:SetTextColor(0.9,0.9,0.9)
			w.menu[mid]:EnableMouse(true)
			w.menu[mid]:SetScript("OnMouseUp", function() if not w.content then return end w.content:SetContentTab(mid) end)
		end
		w.menu[1].text:SetText("Duels in the local memory")
		w.menu[2].text:SetText("Duels in the stored memory")
		w.content = CreateFrame("ScrollFrame", nil, w)
		w.content:SetPoint("TOPLEFT", w.menu[1], "BOTTOMLEFT")
		w.content:SetPoint("BOTTOMRIGHT", -20, 0)
		w.content.inframe = CreateFrame("Frame", nil, w.content)
		w.content.inframe:SetWidth(580)
		w.content.inframe:EnableMouseWheel(true)
		w.content.inframe.ids = {}
		w.content:SetScrollChild(w.content.inframe)
		function w.content:SetContentTab(mid)
			local f = self.inframe
			if f.curid and f.ids[f.curid] then
				for num, frame in ipairs(f.ids[f.curid]) do frame:Hide() end
				w.menu[f.curid].texture:SetTexture(0.5,0.5,0.5,0.5)
			end
			w.menu[mid].texture:SetTexture(0.7,0.7,0.7,0.5)
			-- local
			if mid == 1 then
				if not f.ids[1] then
					f.ids[1] = {}
					local mt = f.ids[1]
					mt[1] = CreateFrame("Frame", nil, f)
					local actions = mt[1]
					actions.texture = actions:CreateTexture(nil, "BACKGROUND")
					actions.texture:SetAllPoints()
					actions.texture:SetTexture(0.7,0.7,0.7,0.5)
					actions.show = CreateFrame("Frame", nil, actions)
					actions.show:EnableMouse(true)
					actions.show.text = actions.show:CreateFontString(nil, "BACKGROUND")
					actions.show.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					actions.show.text:SetTextColor(1,1,1)
					actions.show.text:SetText("Show")
					actions.show.text:SetJustifyH("LEFT")
					actions.show.texture = actions.show:CreateTexture(nil, "BACKGROUND")
					actions.show.texture:SetAllPoints()
					actions.show.texture:SetTexture(0.3,0.3,0.3,0.5)
					actions.show.text:SetPoint("TOPLEFT", f, 15, -15)
					actions.show:SetPoint("TOPLEFT", actions.show.text, "TOPLEFT", -5, 5)
					actions.show:SetPoint("BOTTOMRIGHT", actions.show.text, "BOTTOMRIGHT", 5, -5)
					actions:SetPoint("TOPLEFT", actions.show, -5, 5)
					actions.show:SetScript("OnMouseUp", function()
						if not mt[2].ch then return end
						DueLog:ShowWindow()
						DueLog:GenerateWindowContents(mt[2].ch.num, "casts")
						w:Hide()
					end)
					actions.save = CreateFrame("Frame", nil, actions)
					actions.save:EnableMouse(true)
					actions.save.text = actions.save:CreateFontString(nil, "BACKGROUND")
					actions.save.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					actions.save.text:SetTextColor(1,1,1)
					actions.save.text:SetText("Save")
					actions.save.text:SetJustifyH("LEFT")
					actions.save.texture = actions.save:CreateTexture(nil, "BACKGROUND")
					actions.save.texture:SetAllPoints()
					actions.save.texture:SetTexture(0.3,0.3,0.3,0.5)
					actions.save.text:SetPoint("LEFT", actions.show.text, "RIGHT", 15, 0)
					actions.save:SetPoint("TOPLEFT", actions.save.text, "TOPLEFT", -5, 5)
					actions.save:SetPoint("BOTTOMRIGHT", actions.save.text, "BOTTOMRIGHT", 5, -5)
					actions.save:SetScript("OnMouseUp", function()
						if not mt[2].ch then return end
						if not DueLogDB then DueLogDB={duels={}} end
						tinsert(DueLogDB.duels, DueLog.duels[mt[2].ch.num])
						w.content:SetContentTab(2)
						f.ids[2][2][#DueLogDB.duels]:GetScript("OnMouseUp")(f.ids[2][2][#DueLogDB.duels])
					end)
					actions.send = CreateFrame("Frame", nil, actions)
					actions.send:EnableMouse(true)
					actions.send.text = actions.send:CreateFontString(nil, "BACKGROUND")
					actions.send.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					actions.send.text:SetTextColor(1,1,1)
					actions.send.text:SetText("Send")
					actions.send.text:SetJustifyH("LEFT")
					actions.send.texture = actions.send:CreateTexture(nil, "BACKGROUND")
					actions.send.texture:SetAllPoints()
					actions.send.texture:SetTexture(0.3,0.3,0.3,0.5)
					actions.send.text:SetPoint("LEFT", actions.save.text, "RIGHT", 15, 0)
					actions.send:SetPoint("TOPLEFT", actions.send.text, "TOPLEFT", -5, 5)
					actions.send:SetPoint("BOTTOMRIGHT", actions.send.text, "BOTTOMRIGHT", 5, -5)
					actions.send:SetScript("OnMouseUp", function()
						if not mt[2].ch then return end
						w:Hide()
						DueLog:TransmitWindow()
						DueLog.transmit.window.content:SetContentTab(1)
						DueLog.transmit.window.content.inframe.ids[1][2][mt[2].ch.num]:GetScript("OnMouseUp")(DueLog.transmit.window.content.inframe.ids[1][2][mt[2].ch.num])
					end)
					actions.del = CreateFrame("Frame", nil, actions)
					actions.del:EnableMouse(true)
					actions.del.text = actions.del:CreateFontString(nil, "BACKGROUND")
					actions.del.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					actions.del.text:SetTextColor(1,1,1)
					actions.del.text:SetText("Delete")
					actions.del.text:SetJustifyH("LEFT")
					actions.del.texture = actions.del:CreateTexture(nil, "BACKGROUND")
					actions.del.texture:SetAllPoints()
					actions.del.texture:SetTexture(0.3,0.3,0.3,0.5)
					actions.del.text:SetPoint("LEFT", actions.send.text, "RIGHT", 15, 0)
					actions.del:SetPoint("TOPLEFT", actions.del.text, "TOPLEFT", -5, 5)
					actions.del:SetPoint("BOTTOMRIGHT", actions.del.text, "BOTTOMRIGHT", 5, -5)
					actions.del:SetScript("OnMouseUp", function()
						if not mt[2].ch then return end
						tremove(DueLog.duels, mt[2].ch.num)
						w.content:SetContentTab(1)
						mt[2].ch.texture:SetTexture(0,0,0,0)
						mt[2].ch = nil
					end)
					actions:SetPoint("BOTTOMRIGHT", actions.del, "BOTTOMRIGHT", 5, -5)
					mt[2] = CreateFrame("Frame", nil, f)
					mt[2]:SetPoint("TOPLEFT", actions, "BOTTOMLEFT", 0, -5)
					mt[2]:SetPoint("BOTTOMRIGHT", -10, 10)
				else
					f.ids[1][1]:Show()
					f.ids[1][2]:Show()
				end
				if not DueLog.duels then DueLog.duels = {} end
				local a = 33+40*#DueLog.duels
				f:SetHeight(a)
				if a <= w.content:GetHeight() then w.content.slider:Disable() else w.content.slider:Enable() end
				local mt = f.ids[1]
				for c,fr in ipairs(mt[2]) do
					fr:Hide()
				end
				for num, duel in ipairs(DueLog.duels) do
					if not mt[2][num] then
						mt[2][num] = CreateFrame("Frame", nil, mt[2])
						local of = mt[2][num]
						of.num = num
						of:SetPoint("TOPLEFT", 0, (-40*(num-1)))
						of:SetPoint("TOPRIGHT", 0, (-40*(num-1)))
						of:SetHeight(40)
						of.texture = of:CreateTexture(nil, "BACKGROUND")
						of.texture:SetAllPoints()
						of.texture:SetTexture(0,0,0,0)
						of:EnableMouse(true)
						of:SetScript("OnMouseUp", function(self) if mt[2].ch then mt[2].ch.texture:SetTexture(0,0,0,0) end mt[2].ch = self self.texture:SetTexture(1,1,1,0.5) end)
						of:SetScript("OnEnter", function(self) if not (mt[2].ch and mt[2].ch == self) then self.texture:SetTexture(1,1,1,0.2) end end)
						of:SetScript("OnLeave", function(self) if not (mt[2].ch and mt[2].ch == self) then self.texture:SetTexture(0,0,0,0) end end)
						of.text1 = of:CreateFontString(nil, "OVERLAY")
						of.text1:SetFont("Interface\\Addons\\DueLog\\font.ttf", 12)
						of.text1:SetTextColor(1,0.82,0)
						of.text1:SetPoint("TOPLEFT", 5, -5)
						of.text1:SetPoint("TOPRIGHT", -5, -5)
						of.text1:SetJustifyH("LEFT")
						of.text2 = of:CreateFontString(nil, "OVERLAY")
						of.text2:SetFont("Interface\\Addons\\DueLog\\font.ttf", 8)
						of.text2:SetTextColor(1,1,1)
						of.text2:SetPoint("TOPLEFT", of.text1, "BOTTOMLEFT")
						of.text2:SetPoint("BOTTOMRIGHT", -5, 5)
						of.text2:SetJustifyH("LEFT")
					else
						mt[2][num]:Show()
					end
					mt[2][num].text1:SetText("["..num.."]"..duel.duel1.." - "..duel.duel2)
					mt[2][num].text2:SetText("Duration: "..round(duel.duration, 2).." sec (Winner: "..duel.winner..")")
				end
			--saved
			elseif mid == 2 then
				if not f.ids[2] then
					f.ids[2] = {}
					local mt = f.ids[2]
					mt[1] = CreateFrame("Frame", nil, f)
					local actions = mt[1]
					actions.texture = actions:CreateTexture(nil, "BACKGROUND")
					actions.texture:SetAllPoints()
					actions.texture:SetTexture(0.7,0.7,0.7,0.5)
					actions.loadall = CreateFrame("Frame", nil, actions)
					actions.loadall:EnableMouse(true)
					actions.loadall.text = actions.loadall:CreateFontString(nil, "BACKGROUND")
					actions.loadall.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					actions.loadall.text:SetTextColor(1,1,1)
					actions.loadall.text:SetText("Load all")
					actions.loadall.text:SetJustifyH("LEFT")
					actions.loadall.texture = actions.loadall:CreateTexture(nil, "BACKGROUND")
					actions.loadall.texture:SetAllPoints()
					actions.loadall.texture:SetTexture(0.3,0.3,0.3,0.5)
					actions.loadall.text:SetPoint("TOPLEFT", f, 15, -15)
					actions.loadall:SetPoint("TOPLEFT", actions.loadall.text, "TOPLEFT", -5, 5)
					actions.loadall:SetPoint("BOTTOMRIGHT", actions.loadall.text, "BOTTOMRIGHT", 5, -5)
					actions:SetPoint("TOPLEFT", actions.loadall, -5, 5)
					actions.loadall:SetScript("OnMouseUp", function()
						if not DueLogDB then DueLogDB = {duels={}} end
						if not DueLog.duels then DueLog.duels = {} end
						for key, duel in ipairs(DueLogDB.duels) do
							tinsert(DueLog.duels, duel)
						end
						w.content:SetContentTab(1)
					end)
					actions.load = CreateFrame("Frame", nil, actions)
					actions.load:EnableMouse(true)
					actions.load.text = actions.load:CreateFontString(nil, "BACKGROUND")
					actions.load.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					actions.load.text:SetTextColor(1,1,1)
					actions.load.text:SetText("Load")
					actions.load.text:SetJustifyH("LEFT")
					actions.load.texture = actions.load:CreateTexture(nil, "BACKGROUND")
					actions.load.texture:SetAllPoints()
					actions.load.texture:SetTexture(0.3,0.3,0.3,0.5)
					actions.load.text:SetPoint("LEFT", actions.loadall.text, "RIGHT", 15, 0)
					actions.load:SetPoint("TOPLEFT", actions.load.text, "TOPLEFT", -5, 5)
					actions.load:SetPoint("BOTTOMRIGHT", actions.load.text, "BOTTOMRIGHT", 5, -5)
					actions.load:SetScript("OnMouseUp", function()
						if not mt[2].ch then return end
						if not DueLogDB then DueLogDB={duels={}} end
						if not DueLog.duels then DueLog.duels = {} end
						tinsert(DueLog.duels, DueLogDB.duels[mt[2].ch.num])
						w.content:SetContentTab(1)
						f.ids[1][2][#DueLog.duels]:GetScript("OnMouseUp")(f.ids[1][2][#DueLog.duels])
					end)
					actions.del = CreateFrame("Frame", nil, actions)
					actions.del:EnableMouse(true)
					actions.del.text = actions.del:CreateFontString(nil, "BACKGROUND")
					actions.del.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					actions.del.text:SetTextColor(1,1,1)
					actions.del.text:SetText("Delete")
					actions.del.text:SetJustifyH("LEFT")
					actions.del.texture = actions.del:CreateTexture(nil, "BACKGROUND")
					actions.del.texture:SetAllPoints()
					actions.del.texture:SetTexture(0.3,0.3,0.3,0.5)
					actions.del.text:SetPoint("LEFT", actions.load.text, "RIGHT", 15, 0)
					actions.del:SetPoint("TOPLEFT", actions.del.text, "TOPLEFT", -5, 5)
					actions.del:SetPoint("BOTTOMRIGHT", actions.del.text, "BOTTOMRIGHT", 5, -5)
					actions.del:SetScript("OnMouseUp", function()
						if not mt[2].ch then return end
						tremove(DueLogDB.duels, mt[2].ch.num)
						w.content:SetContentTab(2)
						mt[2].ch.texture:SetTexture(0,0,0,0)
						mt[2].ch = nil
					end)
					actions:SetPoint("BOTTOMRIGHT", actions.del, "BOTTOMRIGHT", 5, -5)
					mt[2] = CreateFrame("Frame", nil, f)
					mt[2]:SetPoint("TOPLEFT", actions, "BOTTOMLEFT", 0, -5)
					mt[2]:SetPoint("BOTTOMRIGHT", -10, 10)
				else
					f.ids[2][1]:Show()
					f.ids[2][2]:Show()
				end
				if not DueLogDB then DueLogDB = {} end
				if not DueLogDB.duels then DueLogDB.duels = {} end
				local a = 33+40*#DueLogDB.duels
				f:SetHeight(a)
				if a <= w.content:GetHeight() then w.content.slider:Disable() else w.content.slider:Enable() end
				local mt = f.ids[2]
				for c,fr in ipairs(mt[2]) do
					fr:Hide()
				end
				for num, duel in ipairs(DueLogDB.duels) do
					if not mt[2][num] then
						mt[2][num] = CreateFrame("Frame", nil, mt[2])
						local of = mt[2][num]
						of.num = num
						of:SetPoint("TOPLEFT", 0, (-40*(num-1)))
						of:SetPoint("TOPRIGHT", 0, (-40*(num-1)))
						of:SetHeight(40)
						of.texture = of:CreateTexture(nil, "BACKGROUND")
						of.texture:SetAllPoints()
						of.texture:SetTexture(0,0,0,0)
						of:EnableMouse(true)
						of:SetScript("OnMouseUp", function(self) if mt[2].ch then mt[2].ch.texture:SetTexture(0,0,0,0) end mt[2].ch = self self.texture:SetTexture(1,1,1,0.5) end)
						of:SetScript("OnEnter", function(self) if not (mt[2].ch and mt[2].ch == self) then self.texture:SetTexture(1,1,1,0.2) end end)
						of:SetScript("OnLeave", function(self) if not (mt[2].ch and mt[2].ch == self) then self.texture:SetTexture(0,0,0,0) end end)
						of.text1 = of:CreateFontString(nil, "OVERLAY")
						of.text1:SetFont("Interface\\Addons\\DueLog\\font.ttf", 12)
						of.text1:SetTextColor(1,0.82,0)
						of.text1:SetPoint("TOPLEFT", 5, -5)
						of.text1:SetPoint("TOPRIGHT", -5, -5)
						of.text1:SetJustifyH("LEFT")
						of.text2 = of:CreateFontString(nil, "OVERLAY")
						of.text2:SetFont("Interface\\Addons\\DueLog\\font.ttf", 8)
						of.text2:SetTextColor(1,1,1)
						of.text2:SetPoint("TOPLEFT", of.text1, "BOTTOMLEFT")
						of.text2:SetPoint("BOTTOMRIGHT", -5, 5)
						of.text2:SetJustifyH("LEFT")
					else
						mt[2][num]:Show()
					end
					mt[2][num].text1:SetText("["..num.."]"..duel.duel1.." - "..duel.duel2)
					mt[2][num].text2:SetText("Duration: "..round(duel.duration, 2).." sec (Winner: "..duel.winner..")")
				end
			end
			f.curid = mid
		end
		w.content.slider = CreateFrame("Slider", nil, w.content)
		w.content.slider:SetPoint("TOPLEFT", w.content, "TOPRIGHT", 0, -10)
		w.content.slider:SetPoint("BOTTOMRIGHT", w, "BOTTOMRIGHT", 0, 10)
		w.content.slider:SetOrientation("VERTICAL")
		local slider = w.content.slider
		slider.texture = slider:CreateTexture(nil, "BACKGROUND")
		slider.texture:SetTexture(0.3,0.3,0.3,0.8)
		slider.texture:SetPoint("TOPLEFT", 0, 10)
		slider.texture:SetPoint("BOTTOMRIGHT", 0, -10)
		slider:SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob")
		slider:SetMinMaxValues(0,1)
		slider.upbutton = CreateFrame("Button", nil, slider)
		slider.upbutton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Up")
		slider.upbutton:SetHighlightTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Highlight")
		slider.upbutton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Down")
		slider.upbutton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Disabled")
		slider.upbutton:RegisterForClicks("LeftButtonUp", "LeftButtonDown")
		slider.upbutton:SetHeight(20)
		slider.upbutton:SetPoint("BOTTOMLEFT", slider, "TOPLEFT", 0, -10)
		slider.upbutton:SetPoint("BOTTOMRIGHT", slider, "TOPRIGHT", 0, -10)
		slider.upbutton:GetNormalTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.upbutton:GetHighlightTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.upbutton:GetPushedTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.upbutton:GetDisabledTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.downbutton = CreateFrame("Button", nil, slider)
		slider.downbutton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Up")
		slider.downbutton:SetHighlightTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Highlight")
		slider.downbutton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
		slider.downbutton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Disabled")
		slider.downbutton:RegisterForClicks("LeftButtonUp", "LeftButtonDown")
		slider.downbutton:SetHeight(20)
		slider.downbutton:SetPoint("TOPLEFT", slider, "BOTTOMLEFT", 0, 10)
		slider.downbutton:SetPoint("TOPRIGHT", slider, "BOTTOMRIGHT", 0, 10)
		slider.downbutton:GetNormalTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.downbutton:GetHighlightTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.downbutton:GetPushedTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.downbutton:GetDisabledTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider:SetScript("OnValueChanged", function(self, value) if value == 1 and self.downbutton:IsEnabled() then self.downbutton:Disable() self.downbutton:SetScript("OnUpdate", nil) else self.downbutton:Enable() end if value == 0 and self.upbutton:IsEnabled() then self.upbutton:Disable() self.upbutton:SetScript("OnUpdate", nil) else self.upbutton:Enable() end w.content:SetVerticalScroll(value*w.content:GetVerticalScrollRange()) end)
		slider:SetScript("OnDisable", function(self) self.downbutton:Disable() self.upbutton:Disable() end)
		slider:SetScript("OnEnable", function(self) if value ~= 1 then self.downbutton:Enable() end if value ~= 0 then self.upbutton:Enable() end end)
		local stepsize = 0.002
		slider.upbutton:SetScript("OnClick", function(self, button, down) if down then self:SetScript("OnUpdate", function() slider:SetValue((slider:GetValue()-stepsize) > 0 and (slider:GetValue()-stepsize) or 0) end) self:SetScript("OnLeave", function(self) self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end) else self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end end)
		slider.downbutton:SetScript("OnClick", function(self, button, down) if down then self:SetScript("OnUpdate", function() slider:SetValue((slider:GetValue()+stepsize) < 1 and (slider:GetValue()+stepsize) or 1) end) self:SetScript("OnLeave", function(self) self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end) else self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end end)
		w.content.inframe:SetScript("OnMouseWheel", function(self, delta) if delta == 1 then slider:SetValue((slider:GetValue() - 0.2) > 0 and (slider:GetValue()-0.2) or 0) else slider:SetValue((slider:GetValue()+0.2) < 1 and (slider:GetValue()+0.2) or 1) end end)
		w.content:SetContentTab((DueLog.duels and #DueLog.duels >= 1) and 1 or 2)
	else
		self.list.window:Show()
	end
	self.list.window.content.slider:SetValue(0)
end

function DueLog:TransmitWindow()
	if not self.transmit then self.transmit = {} end
	if not self.transmit.window then
		self.transmit.window = CreateFrame("Frame", nil, UIParent)
		local w = self.transmit.window
		w:SetPoint("CENTER")
		w:SetWidth(600)
		w:SetHeight(500)
		w:SetFrameStrata("DIALOG")
		w.texture = w:CreateTexture(nil, "BACKGROUND")
		w.texture:SetAllPoints()
		w.texture:SetTexture(0.3,0.3,0.3,0.5)
		w:SetMovable(true)
		w.titlebar = CreateFrame("Frame", nil, w)
		local titlebar = w.titlebar
		titlebar:SetPoint("BOTTOMLEFT", w, "TOPLEFT")
		titlebar:SetPoint("BOTTOMRIGHT", w, "TOPRIGHT")
		titlebar:SetHeight(20)
		titlebar.texture = titlebar:CreateTexture(nil, "BACKGROUND")
		titlebar.texture:SetTexture(0,0,0.5,1)
		titlebar.texture:SetAllPoints()
		titlebar.text = titlebar:CreateFontString(nil)
		titlebar.text:SetPoint("TOPLEFT", 1, 0)
		titlebar.text:SetPoint("BOTTOMRIGHT", 0, 0)
		titlebar.text:SetJustifyH("LEFT")
		titlebar.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", "12")
		titlebar.text:SetTextColor(0.9,0.9,0.9)
		titlebar.text:SetText("DueLog transmission center")
		titlebar.closebutton = CreateFrame("Button", nil, titlebar)
		titlebar:EnableMouse(true)
		titlebar:SetScript("OnMouseDown", function() w:StartMoving() end)
		titlebar:SetScript("OnMouseUp", function() w:StopMovingOrSizing() end)
		local closebutton = titlebar.closebutton
		closebutton:RegisterForClicks("AnyUp")
		closebutton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		closebutton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
		closebutton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
		closebutton:SetScript("OnClick", function() w:Hide() end)
		closebutton:SetPoint("TOPRIGHT", 3, 3)
		closebutton:SetHeight(26)
		closebutton:SetWidth(26)
		closebutton:SetHitRectInsets(6,6,6,6)
		w.menu = {}
		w.menu[1] = CreateFrame("Frame", nil, w)
		w.menu[1]:SetPoint("TOPLEFT")
		w.menu[2] = CreateFrame("Frame", nil, w)
		w.menu[2]:SetPoint("TOP")
		w.menu[3] = CreateFrame("Frame", nil, w)
		w.menu[3]:SetPoint("TOPRIGHT")
		for mid=1, 3 do
			w.menu[mid]:SetWidth(195)
			w.menu[mid]:SetHeight(20)
			w.menu[mid].texture = w.menu[mid]:CreateTexture(nil, "BACKGROUND")
			w.menu[mid].texture:SetTexture(0.5,0.5,0.5,0.5)
			w.menu[mid].texture:SetAllPoints()
			w.menu[mid].text = w.menu[mid]:CreateFontString(nil, "ARTWORK")
			w.menu[mid].text:SetAllPoints()
			w.menu[mid].text:SetFont("Interface\\Addons\\DueLog\\font.ttf", "12")
			w.menu[mid].text:SetTextColor(0.9,0.9,0.9)
			w.menu[mid]:EnableMouse(true)
			w.menu[mid]:SetScript("OnMouseUp", function() if not w.content then return end w.content:SetContentTab(mid) end)
		end
		w.menu[1].text:SetText("Send")
		w.menu[2].text:SetText("Incoming")
		w.menu[3].text:SetText("Outgoing")
		w.content = CreateFrame("ScrollFrame", nil, w)
		w.content:SetPoint("TOPLEFT", w.menu[1], "BOTTOMLEFT")
		w.content:SetPoint("BOTTOMRIGHT", -20, 0)
		w.content.inframe = CreateFrame("Frame", nil, w.content)
		w.content.inframe:SetWidth(580)
		w.content.inframe:EnableMouseWheel(true)
		w.content.inframe.ids = {}
		w.content:SetScrollChild(w.content.inframe)
		function w.content:SetContentTab(mid)
			local f = self.inframe
			if f.curid and f.ids[f.curid] then
				for num, frame in ipairs(f.ids[f.curid]) do frame:Hide() end
				w.menu[f.curid].texture:SetTexture(0.5,0.5,0.5,0.5)
			end
			w.menu[mid].texture:SetTexture(0.7,0.7,0.7,0.5)
			--sending
			if mid == 1 then
				if not f.ids[1] then
					f.ids[1] = {}
					local mt = f.ids[1]
					mt[1] = CreateFrame("Frame", nil, f)
					local name = mt[1]
					name.tag = name:CreateFontString(nil, "ARTWORK")
					name.tag:SetFont("Interface\\Addons\\DueLog\\font.ttf", 16)
					name.tag:SetPoint("TOPLEFT", f, 15, -15)
					name.tag:SetPoint("TOPRIGHT", f, -15, -15)
					name.tag:SetTextColor(1,1,1)
					name.tag:SetJustifyH("LEFT")
					name.tag:SetText("Enter the name of the player you want to send the duel to:")
					name.box = CreateFrame("EditBox", nil, name)
					name.box:SetPoint("TOPLEFT", name.tag, "BOTTOMLEFT", 0, -5)
					name.box:SetPoint("TOPRIGHT", name.tag, "BOTTOMRIGHT", 0, -5)
					name.box:SetHeight(15)
					name.box:SetMultiLine(nil)
					name.box:SetScript("OnEscapePressed", function(self) self:ClearFocus() self:SetText("") end)
					name.box:SetScript("OnEnterPressed", function(self) self:ClearFocus() end)
					name.box:SetFont("Interface\\Addons\\DueLog\\font.ttf", 10)
					name.box:SetTextColor(1,1,1)
					name.box:SetTextInsets(5,5,0,0)
					name.box:SetAutoFocus(false)
					name:SetPoint("TOPLEFT", name.tag, "TOPLEFT", -5, 5)
					name.box.texture = name.box:CreateTexture(nil, "OVERLAY")
					name.box.texture:SetAllPoints()
					name.box.texture:SetTexture(0.1,0.1,0.1,0.5)
					name.texture = name:CreateTexture(nil, "BACKGROUND")
					name.texture:SetAllPoints()
					name.texture:SetTexture(0.7,0.7,0.7,0.5)
					name.send = CreateFrame("Frame", nil, name)
					name.send:EnableMouse(true)
					name.send.text = name.send:CreateFontString(nil, "BACKGROUND")
					name.send.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					name.send.text:SetTextColor(1,1,1)
					name.send.text:SetText("Send")
					name.send.text:SetJustifyH("LEFT")
					name.send.texture = name.send:CreateTexture(nil, "BACKGROUND")
					name.send.texture:SetTexture(0.3,0.3,0.3,0.5)
					name.send.texture:SetAllPoints()
					name.send.text:SetPoint("TOPRIGHT", name.box, "BOTTOMRIGHT", -5, -10)
					name.send:SetPoint("TOPLEFT", name.send.text, "TOPLEFT", -5, 5)
					name.send:SetPoint("BOTTOMRIGHT", name.send.text, "BOTTOMRIGHT", 5, -5)
					name:SetPoint("BOTTOMRIGHT", name.send, "BOTTOMRIGHT", 5, -1*(name.send:GetHeight()+5))
					name.send:SetScript("OnMouseUp", function()
						if (strlen(name.box:GetText()) > 0) and (mt[2].ch) then
							if name.box:GetText() == UnitName("player") then return end
							local d = DueLog.duels[mt[2].ch.num]
							local cid = DueLog:GetAvailableConnectionID()
							SendAddonMessage("DueLog", "request|"..d.duel1.."|"..d.duel2.."|"..d.duration.."|"..d.winner.."|"..d.loser.."|"..d.started.."|"..d.ended.."|"..#d.combatlog.."|"..cid.."|"..(d.version or 40200).."|1|", "WHISPER", name.box:GetText(), true)
							DueLog.connections[cid] = {t=name.box:GetText(), n=mt[2].ch.num, role="SNDR", clog = {}, pct="Waiting", compress=true}
						end
					end)
					name.nocompsend = CreateFrame("Frame", nil, name)
					name.nocompsend:EnableMouse(true)
					name.nocompsend.text = name.nocompsend:CreateFontString(nil, "BACKGROUND")
					name.nocompsend.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 13)
					name.nocompsend.text:SetTextColor(1,1,1)
					name.nocompsend.text:SetText("Send without compression")
					name.nocompsend.text:SetJustifyH("LEFT")
					name.nocompsend.texture = name.nocompsend:CreateTexture(nil, "BACKGROUND")
					name.nocompsend.texture:SetTexture(0.3,0.3,0.3,0.5)
					name.nocompsend.texture:SetAllPoints()
					name.nocompsend.text:SetPoint("RIGHT", name.send.text, "LEFT", -15, 0)
					name.nocompsend:SetPoint("TOPLEFT", name.nocompsend.text, "TOPLEFT", -5, 5)
					name.nocompsend:SetPoint("BOTTOMRIGHT", name.nocompsend.text, "BOTTOMRIGHT", 5, -5)
					name.nocompsend:SetScript("OnMouseUp", function()
						if (strlen(name.box:GetText()) > 0) and (mt[2].ch) then
							if name.box:GetText() == UnitName("player") then return end
							local d = DueLog.duels[mt[2].ch.num]
							local cid = DueLog:GetAvailableConnectionID()
							SendAddonMessage("DueLog", "request|"..d.duel1.."|"..d.duel2.."|"..d.duration.."|"..d.winner.."|"..d.loser.."|"..d.started.."|"..d.ended.."|"..#d.combatlog.."|"..cid.."|".."|"..(d.version or 40200).."|", "WHISPER", name.box:GetText(), true)
							DueLog.connections[cid] = {t=name.box:GetText(), n=mt[2].ch.num, role="SNDR", clog={}, pct="Waiting", num=#d.combatlog}
							for cnum in ipairs(DueLog.duels[mt[2].ch.num].combatlog) do
								DueLog.connections[cid].clog[cnum] = false
							end
						end
					end)
					mt[2] = CreateFrame("Frame", nil, f)
					mt[2]:SetPoint("TOPLEFT", name, "BOTTOMLEFT", 0, -5)
					mt[2]:SetPoint("BOTTOMRIGHT", -10, 10)
				else
					f.ids[1][1]:Show()
					f.ids[1][2]:Show()
				end
				if not DueLog.duels then DueLog.duels = {} end
				local a=f.ids[1][1].tag:GetHeight()+15+(40*#DueLog.duels)
				f:SetHeight(a)
				if a <= w.content:GetHeight() then w.content.slider:Disable() else w.content.slider:Enable() end
				local mt = f.ids[1]
				for c, fr in ipairs(mt[2]) do
					fr:Hide()
				end
				for num, duel in ipairs(DueLog.duels) do
					if not mt[2][num] then
						mt[2][num] = CreateFrame("Frame", nil, mt[2])
						local of = mt[2][num]
						of.num = num
						of:SetPoint("TOPLEFT", 0, (-40*(num-1)))
						of:SetPoint("TOPRIGHT", 0, (-40*(num-1)))
						of:SetHeight(40)
						of.texture = of:CreateTexture(nil, "BACKGROUND")
						of.texture:SetAllPoints()
						of.texture:SetTexture(0,0,0,0)
						of:EnableMouse(true)
						of:SetScript("OnMouseUp", function(self) if mt[2].ch then mt[2].ch.texture:SetTexture(0,0,0,0) end mt[2].ch = self self.texture:SetTexture(1,1,1,0.5) end)
						of:SetScript("OnEnter", function(self) if not (mt[2].ch and mt[2].ch == self) then self.texture:SetTexture(1,1,1,0.2) end end)
						of:SetScript("OnLeave", function(self) if not (mt[2].ch and mt[2].ch == self) then self.texture:SetTexture(0,0,0,0) end end)
						of.text1 = of:CreateFontString(nil, "OVERLAY")
						of.text1:SetFont("Interface\\Addons\\DueLog\\font.ttf", 12)
						of.text1:SetTextColor(1,0.82,0)
						of.text1:SetPoint("TOPLEFT", 5, -5)
						of.text1:SetPoint("TOPRIGHT", -5, -5)
						of.text1:SetJustifyH("LEFT")
						of.text2 = of:CreateFontString(nil, "OVERLAY")
						of.text2:SetFont("Interface\\Addons\\DueLog\\font.ttf", 8)
						of.text2:SetTextColor(1,1,1)
						of.text2:SetPoint("TOPLEFT", of.text1, "BOTTOMLEFT")
						of.text2:SetPoint("BOTTOMRIGHT", -5, 5)
						of.text2:SetJustifyH("LEFT")
					else
						mt[2][num]:Show()
					end
					mt[2][num].text1:SetText("["..num.."] "..duel.duel1.." - "..duel.duel2)
					mt[2][num].text2:SetText("Duration: "..round(duel.duration, 2).." sec (Winner: "..duel.winner..")")
				end
			--incoming overview
			elseif mid == 2 then
				if not f.ids[2] then
					f.ids[2] = {}
					local mt = f.ids[2]
					mt[1] = CreateFrame("Frame", nil, f)
					local main = mt[1]
					main.frames = {}
					main:SetPoint("TOPLEFT", 10, -10)
					main:SetPoint("BOTTOMRIGHT", -10, 10)
					main.texture = main:CreateTexture(nil, "BACKGROUND")
					main.texture:SetAllPoints()
					main.texture:SetTexture(0.1,0.1,0.1,0.5)
				else
					f.ids[2][1]:Show()
				end
				local transmissions = {}
				if not DueLog.connections then DueLog.connections = {} end
				for i,k in pairs(DueLog.connections) do
					if k.role == "RCVR" then
						tinsert(transmissions, {id=i, t=k})
					end
				end
				f:SetHeight(#transmissions*35+35)
				if f:GetHeight() > w.content:GetHeight() then w.content.slider:Enable() else w.content.slider:Disable() end
				for key, frame in ipairs(f.ids[2][1].frames) do
					frame:Hide()
				end
				for num, data in ipairs(transmissions) do
					if not f.ids[2][1].frames[num] then
						f.ids[2][1].frames[num] = CreateFrame("Frame", nil, f.ids[2][1])
						local tf = f.ids[2][1].frames[num]
						if num == 1 then
							tf:SetPoint("TOPLEFT", 10, -10)
						else
							tf:SetPoint("TOPLEFT", f.ids[2][1].frames[num-1], "BOTTOMLEFT", 0, -5)
						end
						tf.texture = tf:CreateTexture(nil, "BACKGROUND")
						tf.texture:SetTexture(0.6,0.6,0.6,0.5)
						tf.texture:SetAllPoints()
						tf.text = tf:CreateFontString(nil, "ARTWORK")
						tf.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 12)
						tf.text:SetTextColor(1,0.82,0)
						tf.text:SetPoint("TOPLEFT", 5, -5)
						tf.text:SetJustifyH("LEFT")
						tf.text2 = tf:CreateFontString(nil, "ARTWORK")
						tf.text2:SetFont("Interface\\Addons\\DueLog\\font.ttf", 8)
						tf.text2:SetTextColor(1,1,1)
						tf.text2:SetPoint("TOPLEFT", tf.text, "BOTTOMLEFT")
						tf.text2:SetPoint("BOTTOMRIGHT", -5, 5)
						tf.text2:SetJustifyH("LEFT")
						tf:SetHeight(30)
						tf:SetWidth(230)
						tf.progressbar = CreateFrame("Frame", nil, tf)
						tf.progressbar:SetScript("OnUpdate", function(self) if not self.texture or not self.text or not tf.conn then return end if (tf.conn.pct or 0) ~= self.texture.cache then self.texture.cache = tf.conn.pct or 0 self.texture:SetWidth(305*((tonumber(tf.conn.pct) and tonumber(tf.conn.pct)>0) and tonumber(tf.conn.pct) or -1)) end local a =(tonumber(tf.conn.pct) and round((tf.conn.pct or 0)*100,0)) or tf.conn.pct if a ~= self.text.cache then self.text.cache = a self.text:SetText(tonumber(a) and a.."%" or a) end end)
						tf.progressbar.text = tf.progressbar:CreateFontString(nil, "ARTWORK")
						tf.progressbar.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 15)
						tf.progressbar.text:SetTextColor(1,1,1)
						tf.progressbar.text:SetAllPoints()
						tf.progressbar:SetPoint("TOPLEFT", tf, "TOPRIGHT", 5, 0)
						tf.progressbar:SetPoint("BOTTOMLEFT", tf, "BOTTOMRIGHT", 5, 0)
						tf.progressbar:SetWidth(305)
						tf.progressbar.texture = tf.progressbar:CreateTexture(nil, "BACKGROUND")
						tf.progressbar.texture:SetTexture(0,0,1,0.8)
						tf.progressbar.texture:SetPoint("TOPLEFT")
						tf.progressbar.texture:SetPoint("BOTTOMLEFT")
					else
						f.ids[2][1].frames[num]:Show()
					end
					f.ids[2][1].frames[num].conn = data.t
					f.ids[2][1].frames[num].text:SetText("["..data.id.."] "..data.t.duel.duel1.." - "..data.t.duel.duel2)
					f.ids[2][1].frames[num].text2:SetText("Sender: "..data.t.t)
				end
			--outgoing overview
			elseif mid == 3 then
				if not f.ids[3] then
					f.ids[3] = {}
					local mt = f.ids[3]
					mt[1] = CreateFrame("Frame", nil, f)
					local main = mt[1]
					main.frames = {}
					main:SetPoint("TOPLEFT", 10, -10)
					main:SetPoint("BOTTOMRIGHT", -10, 10)
					main.texture = main:CreateTexture(nil, "BACKGROUND")
					main.texture:SetAllPoints()
					main.texture:SetTexture(0.1,0.1,0.1,0.5)
				else
					f.ids[3][1]:Show()
				end
				local transmissions = {}
				if not DueLog.connections then DueLog.connections = {} end
				for i,k in pairs(DueLog.connections) do
					if k.role == "SNDR" then
						tinsert(transmissions, {id=i, t=k})
					end
				end
				f:SetHeight(#transmissions*35+35)
				if f:GetHeight() > w.content:GetHeight() then
					w.content.slider:Enable()
				else
					w.content.slider:Disable()
				end
				for key, frame in ipairs(f.ids[3][1].frames) do
					frame:Hide()
				end
				for num, data in ipairs(transmissions) do
					if not f.ids[3][1].frames[num] then
						f.ids[3][1].frames[num] = CreateFrame("Frame", nil, f.ids[3][1])
						local tf = f.ids[3][1].frames[num]
						if num == 1 then
							tf:SetPoint("TOPLEFT", 10, -10)
						else
							tf:SetPoint("TOPLEFT", f.ids[3][1].frames[num-1], "BOTTOMLEFT", 0, -5)
						end
						tf.texture = tf:CreateTexture(nil, "BACKGROUND")
						tf.texture:SetTexture(0.6,0.6,0.6,0.5)
						tf.texture:SetAllPoints()
						tf.text = tf:CreateFontString(nil, "ARTWORK")
						tf.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 12)
						tf.text:SetTextColor(1,0.82,0)
						tf.text:SetPoint("TOPLEFT", 5, -5)
						tf.text:SetJustifyH("LEFT")
						tf.text2 = tf:CreateFontString(nil, "ARTWORK")
						tf.text2:SetFont("Interface\\Addons\\DueLog\\font.ttf", 8)
						tf.text2:SetTextColor(1,1,1)
						tf.text2:SetPoint("TOPLEFT", tf.text, "BOTTOMLEFT")
						tf.text2:SetPoint("BOTTOMRIGHT", -5, 5)
						tf.text2:SetJustifyH("LEFT")
						tf:SetHeight(30)
						tf:SetWidth(230)
						tf.progressbar = CreateFrame("Frame", nil, tf)
						tf.progressbar:SetScript("OnUpdate", function(self) if not self.texture or not self.text or not tf.conn then return end if (tf.conn.pct or 0) ~= self.texture.cache then self.texture.cache = tf.conn.pct or 0 self.texture:SetWidth(305*(tonumber(tf.conn.pct) or 0)) end local a =(tonumber(tf.conn.pct) and round((tf.conn.pct or 0)*100,0)) or tf.conn.pct if a ~= self.text.cache then self.text.cache = a self.text:SetText(tonumber(a) and a.."%" or a) end end)
						tf.progressbar.text = tf.progressbar:CreateFontString(nil, "ARTWORK")
						tf.progressbar.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 15)
						tf.progressbar.text:SetTextColor(1,1,1)
						tf.progressbar.text:SetAllPoints()
						tf.progressbar:SetPoint("TOPLEFT", tf, "TOPRIGHT", 5, 0)
						tf.progressbar:SetPoint("BOTTOMLEFT", tf, "BOTTOMRIGHT", 5, 0)
						tf.progressbar:SetWidth(305)
						tf.progressbar.texture = tf.progressbar:CreateTexture(nil, "BACKGROUND")
						tf.progressbar.texture:SetTexture(0,0,1,0.8)
						tf.progressbar.texture:SetPoint("TOPLEFT")
						tf.progressbar.texture:SetPoint("BOTTOMLEFT")
					else
						f.ids[3][1].frames[num]:Show()
					end
					f.ids[3][1].frames[num].conn = data.t
					f.ids[3][1].frames[num].text:SetText("["..data.id.."] "..DueLog.duels[data.t.n].duel1.." - "..DueLog.duels[data.t.n].duel2)
					f.ids[3][1].frames[num].text2:SetText("To: "..data.t.t)
				end
			end
			f.curid = mid
		end
		w.content.slider = CreateFrame("Slider", nil, w.content)
		w.content.slider:SetPoint("TOPLEFT", w.content, "TOPRIGHT", 0, -10)
		w.content.slider:SetPoint("BOTTOMRIGHT", w, "BOTTOMRIGHT", 0, 10)
		w.content.slider:SetOrientation("VERTICAL")
		local slider = w.content.slider
		slider.texture = slider:CreateTexture(nil, "BACKGROUND")
		slider.texture:SetTexture(0.3,0.3,0.3,0.8)
		slider.texture:SetPoint("TOPLEFT", 0, 10)
		slider.texture:SetPoint("BOTTOMRIGHT", 0, -10)
		slider:SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob")
		slider:SetMinMaxValues(0,1)
		slider.upbutton = CreateFrame("Button", nil, slider)
		slider.upbutton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Up")
		slider.upbutton:SetHighlightTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Highlight")
		slider.upbutton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Down")
		slider.upbutton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Disabled")
		slider.upbutton:RegisterForClicks("LeftButtonUp", "LeftButtonDown")
		slider.upbutton:SetHeight(20)
		slider.upbutton:SetPoint("BOTTOMLEFT", slider, "TOPLEFT", 0, -10)
		slider.upbutton:SetPoint("BOTTOMRIGHT", slider, "TOPRIGHT", 0, -10)
		slider.upbutton:GetNormalTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.upbutton:GetHighlightTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.upbutton:GetPushedTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.upbutton:GetDisabledTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.downbutton = CreateFrame("Button", nil, slider)
		slider.downbutton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Up")
		slider.downbutton:SetHighlightTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Highlight")
		slider.downbutton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down")
		slider.downbutton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Disabled")
		slider.downbutton:RegisterForClicks("LeftButtonUp", "LeftButtonDown")
		slider.downbutton:SetHeight(20)
		slider.downbutton:SetPoint("TOPLEFT", slider, "BOTTOMLEFT", 0, 10)
		slider.downbutton:SetPoint("TOPRIGHT", slider, "BOTTOMRIGHT", 0, 10)
		slider.downbutton:GetNormalTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.downbutton:GetHighlightTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.downbutton:GetPushedTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider.downbutton:GetDisabledTexture():SetTexCoord(0.2000000029802322,0.25,0.2000000029802322,0.75,0.800000011920929,0.25,0.800000011920929,0.75)
		slider:SetScript("OnValueChanged", function(self, value) if value == 1 and self.downbutton:IsEnabled() then self.downbutton:Disable() self.downbutton:SetScript("OnUpdate", nil) else self.downbutton:Enable() end if value == 0 and self.upbutton:IsEnabled() then self.upbutton:Disable() self.upbutton:SetScript("OnUpdate", nil) else self.upbutton:Enable() end w.content:SetVerticalScroll(value*w.content:GetVerticalScrollRange()) end)
		slider:SetScript("OnDisable", function(self) self.downbutton:Disable() self.upbutton:Disable() end)
		slider:SetScript("OnEnable", function(self) if value ~= 1 then self.downbutton:Enable() end if value ~= 0 then self.upbutton:Enable() end end)
		local stepsize = 0.002
		slider.upbutton:SetScript("OnClick", function(self, button, down) if down then self:SetScript("OnUpdate", function() slider:SetValue((slider:GetValue()-stepsize) > 0 and (slider:GetValue()-stepsize) or 0) end) self:SetScript("OnLeave", function(self) self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end) else self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end end)
		slider.downbutton:SetScript("OnClick", function(self, button, down) if down then self:SetScript("OnUpdate", function() slider:SetValue((slider:GetValue()+stepsize) < 1 and (slider:GetValue()+stepsize) or 1) end) self:SetScript("OnLeave", function(self) self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end) else self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end end)
		w.content.inframe:SetScript("OnMouseWheel", function(self, delta) if delta == 1 then slider:SetValue((slider:GetValue() - 0.2) > 0 and (slider:GetValue()-0.2) or 0) else slider:SetValue((slider:GetValue()+0.2) < 1 and (slider:GetValue()+0.2) or 1) end end)
		w.content:SetContentTab(1)
	else
		self.transmit.window:Show()
	end
	self.transmit.window.content.slider:SetValue(0)
end

function DueLog:ShowWindow()
	if not self.window then
		self.window = CreateFrame("ScrollFrame", nil, UIParent)
		local f = self.window
		f:SetFrameStrata("DIALOG")
		f:SetWidth(460)
		f:SetHeight(200)
		f.texture = f:CreateTexture(nil, "BACKGROUND")
		f.texture:SetTexture(0.3,0.3,0.3,0.5)
		f.texture:SetAllPoints()
		f:SetPoint("CENTER", 0, -10)
		f:SetMovable(true)
		f.titlebar = CreateFrame("Frame", nil, f)
		local titlebar = f.titlebar
		titlebar:SetPoint("BOTTOMLEFT", f, "TOPLEFT")
		titlebar:SetPoint("BOTTOMRIGHT", f, "TOPRIGHT")
		titlebar:SetHeight(20)
		titlebar.texture = titlebar:CreateTexture(nil, "BACKGROUND")
		titlebar.texture:SetTexture(0,0,0.5,1)
		titlebar.texture:SetAllPoints()
		titlebar.text = titlebar:CreateFontString(nil)
		titlebar.text:SetPoint("TOPLEFT", 3, -3)
		titlebar.text:SetPoint("BOTTOMRIGHT", -17, 3)
		titlebar.text:SetJustifyH("LEFT")
		titlebar.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", "12")
		titlebar.text:SetTextColor(0.9,0.9,0.9)
		titlebar.closebutton = CreateFrame("Button", nil, titlebar)
		titlebar:EnableMouse(true)
		titlebar:SetScript("OnMouseDown", function() f:StartMoving() end)
		titlebar:SetScript("OnMouseUp", function() f:StopMovingOrSizing() end)
		local closebutton = titlebar.closebutton
		closebutton:RegisterForClicks("AnyUp")
		closebutton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
		closebutton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
		closebutton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
		closebutton:SetScript("OnClick", function() f:Hide() end)
		closebutton:SetPoint("TOPRIGHT", 3, 3)
		closebutton:SetHeight(26)
		closebutton:SetWidth(26)
		closebutton:SetHitRectInsets(6,6,6,6)
		f.content = CreateFrame("Frame", nil, f)
		local content = f.content
		content:SetHeight(480)
		content.secondbars = {}
		content.icons = {}
		f:SetScrollChild(content)
		f.slider = CreateFrame("Slider", nil, f)
		local slider = f.slider
		slider:SetPoint("TOPLEFT", f, "BOTTOMLEFT", 10, 0)
		slider:SetPoint("TOPRIGHT", f, "BOTTOMRIGHT", -10, 0)
		slider:SetHeight(20)
		slider:SetOrientation("HORIZONTAL")
		slider.texture = slider:CreateTexture(nil, "BACKGROUND")
		slider.texture:SetTexture(0.3,0.3,0.3,0.8)
		slider.texture:SetPoint("BOTTOMRIGHT", 10, 0)
		slider.texture:SetPoint("TOPLEFT", -10, 0)
		slider:SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob")
		slider:GetThumbTexture():SetRotation(math.pi/2)
		slider:SetMinMaxValues(0,1)
		slider.rightbutton = CreateFrame("Button", nil, slider)
		slider.rightbutton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Up")
		slider.rightbutton:SetHighlightTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Highlight")
		slider.rightbutton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Down")
		slider.rightbutton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Disabled")
		slider.rightbutton:GetNormalTexture():SetTexCoord(0.20000000298023, 0.75, 0.8000000192093, 0.75, 0.20000000298023, 0.25, 0.80000001192093, 0.25)
		slider.rightbutton:GetHighlightTexture():SetTexCoord(0.20000000298023, 0.75, 0.8000000192093, 0.75, 0.20000000298023, 0.25, 0.80000001192093, 0.25)
		slider.rightbutton:GetPushedTexture():SetTexCoord(0.20000000298023, 0.75, 0.8000000192093, 0.75, 0.20000000298023, 0.25, 0.80000001192093, 0.25)
		slider.rightbutton:GetDisabledTexture():SetTexCoord(0.20000000298023, 0.75, 0.8000000192093, 0.75, 0.20000000298023, 0.25, 0.80000001192093, 0.25)
		slider.rightbutton:RegisterForClicks("LeftButtonUp", "LeftButtonDown")
		slider.rightbutton:SetWidth(20)
		slider.rightbutton:SetPoint("TOPLEFT", slider, "TOPRIGHT", -10, 0)
		slider.rightbutton:SetPoint("BOTTOMLEFT", slider, "BOTTOMRIGHT", -10, 0)
		slider.leftbutton = CreateFrame("Button", nil, slider)
		slider.leftbutton:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Up")
		slider.leftbutton:SetHighlightTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Highlight")
		slider.leftbutton:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Down")
		slider.leftbutton:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollUpButton-Disabled")
		slider.leftbutton:GetNormalTexture():SetTexCoord(0.80000001192093, 0.25, 0.20000000298023, 0.25, 0.8000000192093, 0.75, 0.20000000298023, 0.75)
		slider.leftbutton:GetHighlightTexture():SetTexCoord(0.80000001192093, 0.25, 0.20000000298023, 0.25, 0.8000000192093, 0.75, 0.20000000298023, 0.75)
		slider.leftbutton:GetPushedTexture():SetTexCoord(0.80000001192093, 0.25, 0.20000000298023, 0.25, 0.8000000192093, 0.75, 0.20000000298023, 0.75)
		slider.leftbutton:GetDisabledTexture():SetTexCoord(0.80000001192093, 0.25, 0.20000000298023, 0.25, 0.8000000192093, 0.75, 0.20000000298023, 0.75)
		slider.leftbutton:RegisterForClicks("LeftButtonUp", "LeftButtonDown")
		slider.leftbutton:SetWidth(20)
		slider.leftbutton:SetPoint("TOPRIGHT", slider, "TOPLEFT", 10, 0)
		slider.leftbutton:SetPoint("BOTTOMRIGHT", slider, "BOTTOMLEFT", 10, 0)
		slider:SetScript("OnValueChanged", function(self, value) if value==1 and self.rightbutton:IsEnabled() then self.rightbutton:Disable() self.rightbutton:SetScript("OnUpdate", nil) else self.rightbutton:Enable() end if value==0 and self.leftbutton:IsEnabled() then self.leftbutton:Disable() self.leftbutton:SetScript("OnUpdate", nil) else self.leftbutton:Enable() end f:SetHorizontalScroll(value*f:GetHorizontalScrollRange()) end)
		local stepsize = 0.002
		slider.leftbutton:SetScript("OnClick", function(self, button, down) if down then self:SetScript("OnUpdate", function() slider:SetValue((slider:GetValue()-stepsize) >=0 and (slider:GetValue()-stepsize) or 0) end) self:SetScript("OnLeave", function(self) self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end) else self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end end)
		slider.rightbutton:SetScript("OnClick", function(self, button, down) if down then self:SetScript("OnUpdate", function() slider:SetValue((slider:GetValue()+stepsize) <=1 and (slider:GetValue()+stepsize) or 1) end) self:SetScript("OnLeave", function() self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end) else self:SetScript("OnUpdate", nil) self:SetScript("OnLeave", nil) end end)
	else
		self.window:Show()
	end
	self.window.slider:SetValue(0)
end

function DueLog:GenerateWindowContents(duelid, type)
	local valid_types = {casts=true}
	if not valid_types[type] then error("Invalid type "..type.." specified") end
	local d = self.duels[duelid]
	local f = self.window
	if type=="casts" then
		f.content:SetWidth(50*d.duration)
		f.titlebar.text:SetText(d.duel1.." - "..d.duel2.." (Duration: "..math.floor(d.duration/600)..(math.floor(d.duration/60)%10)..":"..math.floor((d.duration%60)/10)..math.floor((d.duration%60)%10)..")")
		for num,bar in ipairs(f.content.secondbars) do
			bar:Hide()
		end
		if not f.content.htitle then
			f.content.htitle = {rows={}}
			f.content.htitle.bg = CreateFrame("Frame", nil, f.content)
			local bg = f.content.htitle.bg
			bg:SetPoint("TOPLEFT")
			bg:SetPoint("BOTTOMLEFT")
			bg:SetWidth(100)
			bg.texture = bg:CreateTexture(nil, "ARTWORK")
			bg.texture:SetTexture(0.1,0.1,0.1,0.5)
			bg.texture:SetAllPoints()
		end
		if not f.content.vtitle then
			f.content.vtitle = {}
			f.content.vtitle.bg = CreateFrame("Frame", nil, f.content)
			local bg = f.content.vtitle.bg
			bg:SetPoint("TOPLEFT", f.content.htitle.bg, "TOPRIGHT")
			bg:SetPoint("BOTTOMRIGHT", f.content, "TOPRIGHT", 100, -20)
			bg.texture = bg:CreateTexture(nil, "ARTWORK")
			bg.texture:SetTexture(0.1,0.1,0.1,0.5)
			bg.texture:SetAllPoints()
		end
		for i=1, math.floor(d.duration) do
			if not f.content.secondbars[i] then
				f.content.secondbars[i] = CreateFrame("Frame", nil, f.content)
				local bf = f.content.secondbars[i]
				bf:SetPoint("TOP", f.content, "TOPLEFT", 100+50*i, 0)
				bf:SetPoint("BOTTOM", f.content, "BOTTOMLEFT", 100+50*i, 0)
				bf:SetWidth(1)
				local b=bf:CreateTexture(nil, "OVERLAY")
				b:SetAllPoints()
				b:SetTexture(1,1,1,0.2)
				local text = bf:CreateFontString(nil, "ARTWORK")
				text:SetPoint("TOP", bf, "TOP", 0, -5)
				text:SetFont("Fonts\\FRIZQT__.TTF", 10, "")
				text:SetTextColor(1,1,1,0.5)
				text:SetText(math.floor(i/600)..(math.floor(i/60)%10)..":"..math.floor((i%60)/10)..math.floor((i%60)%10))
			else
				f.content.secondbars[i]:Show()
			end
		end
		local casts, damage, healing, debuffs, absorbs = {[d.duel1]={}, [d.duel2]={}}, {}, {}, {[d.duel1]={}, [d.duel2]={}}, {}
		local subEventEvent = clEvent(d.version,"subevent")
		local srcEvent = clEvent(d.version,"srcname")
		local dstEvent = clEvent(d.version,"dstname")
		local idEvent = clEvent(d.version,"spid")
		local nameEvent = clEvent(d.version,"spname")
		local amountEvent = clEvent(d.version,"spAmount")
		local absorbEvent = clEvent(d.version,"spAbsorbAmt")
		local criticalEvent = clEvent(d.version,"spCriticalDmg")
		for num,data in ipairs(d.combatlog) do
			if ((data[srcEvent] == d.duel1) or (data[srcEvent] == d.duel2)) or (data[dstEvent] and (data[dstEvent] == d.duel1) or (data[dstEvent] == d.duel2)) then
				if data[subEventEvent] == "SPELL_CAST_SUCCESS" then
					if not casts[data[srcEvent]] then casts[data[srcEvent]] = {} end
					tinsert(casts[data[srcEvent]], {icon=(select(3, GetSpellInfo(data[idEvent]))), time=data.time, id=data[idEvent]})
					if not casts[data[srcEvent]].recent_finished_cast then casts[data[srcEvent]].recent_finished_cast = {} end
					casts[data[srcEvent]].recent_finished_cast[data[idEvent]] = casts[data[srcEvent]][#casts[data[srcEvent]]]
				elseif data[subEventEvent] == "SPELL_CAST_FAILED" then
					if not casts[data[srcEvent]] then casts[data[srcEvent]] = {} end
				elseif data[subEventEvent] == "SPELL_CAST_START" then
					if not casts[data[srcEvent]] then casts[data[srcEvent]] = {} end
					tinsert(casts[data[srcEvent]], {icon=(select(3, GetSpellInfo(data[idEvent]))), time=data.time, id=data[idEvent]})
					if not casts[data[srcEvent]].recent_cast then casts[data[srcEvent]].recent_cast = {} end
					if not casts[data[srcEvent]].recent_cast[data[nameEvent]] then casts[data[srcEvent]].recent_cast[data[nameEvent]] = {} end
					casts[data[srcEvent]].recent_cast[data[nameEvent]].t = casts[data[srcEvent]][#casts[data[srcEvent]]]
				elseif data[subEventEvent] == "UNIT_SPELLCAST_SUCCEEDED" then
					if not ((not casts[data[srcEvent]]) or (not casts[data[srcEvent]].recent_cast) or (not casts[data[srcEvent]].recent_cast[data[nameEvent]])) then
						if not casts[data[srcEvent]].recent_finished_cast then casts[data[srcEvent]].recent_finished_cast ={} end
						casts[data[srcEvent]].recent_finished_cast[casts[data[srcEvent]].recent_cast[data[nameEvent]].t.id] = casts[data[srcEvent]].recent_cast[data[nameEvent]].t
						casts[data[srcEvent]].recent_cast[data[nameEvent]] = nil
					end
				elseif data[subEventEvent] == "RANGE_DAMAGE" or data[subEventEvent] == "SPELL_DAMAGE" or data[subEventEvent] == "SPELL_BUILDING_DAMAGE" then
					if not ((not casts[data[srcEvent]]) or (not casts[data[srcEvent]].recent_finished_cast) or (not casts[data[srcEvent]].recent_finished_cast[data[idEvent]])) then
						local c = casts[data[srcEvent]].recent_finished_cast[data[idEvent]]
						c.damage = (c.damage or 0)+data[amountEvent]
						c.critical_dmg = data[criticalEvent]
						damage[data[srcEvent]] = (damage[data[srcEvent]] or 0)+data[amountEvent]
						absorbs[data[dstEvent]] = (absorbs[data[dstEvent]] or 0)+(data[absorbEvent] or 0)
					end
				elseif data[subEventEvent] == "SPELL_PERIODIC_DAMAGE" then
					if not ((not casts[data[srcEvent]]) or (not casts[data[srcEvent]].recent_finished_cast) or (not casts[data[srcEvent]].recent_finished_cast[data[idEvent]])) then
						local c = casts[data[srcEvent]].recent_finished_cast[data[idEvent]]
						c.damage = (c.damage or 0)+data[amountEvent]
						damage[data[srcEvent]] = (damage[data[srcEvent]] or 0)+data[amountEvent]
						absorbs[data[dstEvent]] = (absorbs[data[dstEvent]] or 0)+(data[absorbEvent] or 0)
					end
				elseif data[subEventEvent] == "SWING_DAMAGE" then
					damage[data[srcEvent]] = (damage[data[srcEvent]] or 0)+data[clEvent(d.version,"swingAmount")]
					absorbs[data[dstEvent]] = (absorbs[data[dstEvent]] or 0)+(data[clEvent(d.version,"swingAbsorbAmt")] or 0)
				elseif data[subEventEvent] == "SPELL_HEAL" or data[subEventEvent] == "SPELL_BUILDING_HEAL" then
					if not ((not casts[data[srcEvent]]) or (not casts[data[srcEvent]].recent_finished_cast) or (not casts[data[srcEvent]].recent_finished_cast[data[idEvent]])) then
						local c = casts[data[srcEvent]].recent_finished_cast[data[idEvent]]
						c.healing = (c.healing or 0)+data[amountEvent]
						c.critical_heal = data[amountEvent]
						healing[data[srcEvent]] = (healing[data[srcEvent]] or 0)+data[amountEvent]
					end
				elseif data[subEventEvent] == "SPELL_PERIODIC_HEAL" then
					if not ((not casts[data[srcEvent]]) or (not casts[data[srcEvent]].recent_finished_cast) or (not casts[data[srcEvent]].recent_finished_cast[data[dstEvent]])) then
						local c = casts[data[srcEvent]].recent_finished_cast[data[idEvent]]
						c.healing = (c.healing or 0)+data[amountEvent]
						healing[data[srcEvent]] = (healing[data[srcEvent]] or 0)+data[amountEvent]
					end
				elseif data[subEventEvent] == "SPELL_AURA_APPLIED" then
					if not data[srcEvent] then print(false, "Missing source GUID in combat log event ID "..num..", substituting dest GUID") data[srcEvent] = data[dstEvent] end
					if not debuffs[data[srcEvent]] then debuffs[data[srcEvent]] = {} end
					if not debuffs[data[srcEvent]][data[dstEvent]] then debuffs[data[srcEvent]][data[dstEvent]] = {} end
					local dt = debuffs[data[srcEvent]][data[dstEvent]]
					if not dt[data[nameEvent]] then dt[data[nameEvent]] = {} end
					dt[data[nameEvent]].starttime = dt[data[nameEvent]].starttime or data.time
				elseif data[subEventEvent] == "SPELL_AURA_REMOVED" then
					if not data[srcEvent] then print(false, "Missing source GUID in combat log event ID "..num..", substituting dest GUID") data[srcEvent] = data[dstEvent] end
					if not debuffs[data[srcEvent]] then debuffs[data[srcEvent]] = {} end
					if not debuffs[data[srcEvent]][data[dstEvent]] then debuffs[data[srcEvent]][data[dstEvent]] = {} end
					local dt = debuffs[data[srcEvent]][data[dstEvent]][data[nameEvent]]
					if not dt then debuffs[data[srcEvent]][data[dstEvent]][data[nameEvent]] = {} dt = debuffs[data[srcEvent]][data[dstEvent]][data[nameEvent]] end
					dt.uptime = (dt.uptime or 0)+(data.time-(dt.starttime or d.started))
					dt.starttime = nil
				end
			end
		end
		for num, frame in ipairs(f.content.htitle.rows) do frame:Hide() end
		local c=0
		local numicons=0
		for q,w in ipairs(f.content.icons) do
			w:Hide()
		end
		for i, castlist in pairs(casts) do
			c=c+1
			local r
			if not f.content.htitle.rows[c] then
				f.content.htitle.rows[c]=CreateFrame("Frame", nil, f.content.htitle.bg)
				r = f.content.htitle.rows[c]
				r:SetPoint("TOPLEFT", f.content.htitle.bg, "TOPLEFT", 0, (c-1)*-25-20)
				r:SetPoint("BOTTOMRIGHT", f.content.htitle.bg, "TOPRIGHT", 0, c*-25-20)
				r.texture = r:CreateTexture(nil, "BACKGROUND")
				r.texture:SetAllPoints()
				r.text = r:CreateFontString(nil, "ARTWORK")
				r.text:SetFont("Interface\\Addons\\DueLog\\font.ttf", 15, "")
				r.text:SetTextColor(1,1,1,0.6)
				r.text:SetJustifyH("LEFT")
				r.text:SetPoint("LEFT")
				r.textmouseover = CreateFrame("Frame", nil, r)
				r.textmouseover:EnableMouse(true)
				r.textmouseover:SetAllPoints(r.text)
			else
				r=f.content.htitle.rows[c]
				r:Show()
			end
			if not d.buffcolors then
				d.buffcolors = {ff0000 = true, ["00ff00"] = true, ["0000ff"] = true, ffff00 = true, ["00ffff"] = true, ["44ff44"]=true}
			end
			local buffcolors = d.buffcolors
			if not buffcolors[i] then while true do local h = rndhex() if buffcolors[h] then break else buffcolors[i] = h buffcolors[h] = i end end end
			local b = buffcolors[i]
			r.texture:SetTexture(tonumber("0x"..string.sub(b,1,2))/255, tonumber("0x"..string.sub(b,3,4))/255, tonumber("0x"..string.sub(b,5,6))/255, 0.5)
			r.text:SetText(i)
			r.textmouseover:SetScript("OnEnter", function(self)
				GameTooltip:ClearAllPoints()
				GameTooltip:ClearLines()
				GameTooltip:SetOwner(self, "ANCHOR_NONE")
				GameTooltip:AddLine(i, tonumber("0x"..string.sub(b,1,2))/255, tonumber("0x"..string.sub(b,3,4))/255, tonumber("0x"..string.sub(b,5,6))/255)
				GameTooltip:AddLine("Damage", 1, 0, 0) GameTooltip:AddLine(tostring(damage[i] or 0),1,1,1)
				GameTooltip:AddLine("Healing and absorbs", 0.4, 0.4) GameTooltip:AddDoubleLine("Healing",tostring(healing[i] or 0),1,1,1,1,1,1)
				GameTooltip:AddDoubleLine("Absorbs", tostring(absorbs[i] or 0),1,1,1,1,1,1)
				GameTooltip:AddLine("Uptime", 0, 1, 0)
				for target, bufftable in pairs(debuffs[i] or {}) do
					if bufftable then
						for buff,buffdata in pairs(bufftable) do
							if not buffcolors[target] then
								while true do
									local h = rndhex()
									if not buffcolors[h] then
										buffcolors[target] = h buffcolors[h] = target
										break
									end
								end
							end
							if buffdata and buffdata.uptime then
								GameTooltip:AddDoubleLine("|cffffff00"..buff.."|r on |cff"..buffcolors[target]..target.."|r", round((buffdata.uptime/d.duration)*100, 2).."%", 1, 1, 1, 1, 1, 1)
							end
						end
					end
				end
				GameTooltip:SetPoint("TOP", self, "BOTTOM") GameTooltip:Show()
			end)
			r.textmouseover:SetScript("OnLeave", function() GameTooltip:Hide() end)
			local precast = 0
			for num, castinfo in ipairs(castlist) do
				numicons = numicons+1
				local ic
				if not f.content.icons[numicons] then
					f.content.icons[numicons] = CreateFrame("Frame", nil, r)
					ic = f.content.icons[numicons]
					ic:SetWidth(25)
					ic:SetHeight(25)
					ic:SetFrameStrata("HIGH")
				else
					ic = f.content.icons[numicons]
					ic:Show()
				end
				ic:ClearAllPoints()
				if (castinfo.time-d.started) >= 0 then
					ic:SetPoint("CENTER", r, "RIGHT", 50*(castinfo.time-d.started), 0)
				else
					ic:SetPoint("LEFT", r.text, "RIGHT", 5+(precast*10), 0)
					precast = precast+1
				end
				ic:SetBackdrop({bgFile=castinfo.icon,edgeFile=false, tile=false, edgeSize=0, insets={top=0,bottom=0,left=0,right=0}})
				ic:EnableMouse(true)
				ic:SetScript("OnEnter", function(self) GameTooltip:ClearAllPoints() GameTooltip:ClearLines() GameTooltip:SetOwner(self, "ANCHOR_NONE") GameTooltip:SetHyperlink("spell:"..castinfo.id) if castinfo.damage and (castinfo.damage > 0) then GameTooltip:AddLine("Did "..castinfo.damage.." damage"..(castinfo.critical_dmg and " (Critical)" or ""), 0.8,0,0) end if castinfo.healing and (castinfo.healing > 0) then GameTooltip:AddLine("Did "..castinfo.healing.." healing"..(castinfo.critical_heal and " (Critical)" or ""), 0.4,0.4,1) end GameTooltip:SetPoint("TOPLEFT", self, "CENTER") GameTooltip:Show() end)
				ic:SetScript("OnLeave", function() GameTooltip:Hide() end)
			end
		end
	end
end
SlashCmdList["DUELOG"] = function(input)
	local command, args = string.match(input, "(%S+)%s+(%S+)") if not args then args="" end
	if not command then command = input end
	--[[if command == "show" then
		if (not DueLog.duels) or (not DueLog.duels[tonumber(args)]) then print(true, "Duel "..args.." not found") return end
		DueLog:ShowWindow() DueLog:GenerateWindowContents(tonumber(args), "casts")
	else--]]if command == "list" then
		--[[local d, d2
		if (DueLog.duels) and (#DueLog.duels > 0) then
			print(true, "== CURRENT DUELS ==")
			for num, duelinfo in ipairs(DueLog.duels) do
				print(true, "["..num.."] "..duelinfo.duel1.." - "..duelinfo.duel2.." (Winner: "..duelinfo.winner..") Duration: "..round(duelinfo.duration,2).." seconds")
			end
			d = true
		end
		if (DueLogDB) and (#DueLogDB.duels > 0) then
			print(true, "== SAVED DUELS ==")
			for num, duelinfo in ipairs(DueLogDB.duels) do
				print(true, "["..num.."] "..duelinfo.duel1.." - "..duelinfo.duel2.." (Winner: "..duelinfo.winner..") Duration: "..round(duelinfo.duration,2).." seconds")
			end
			d2 = true
		end
		if not (d or d2) then
			print(true, "No duels available")
		end--]]
		DueLog:ListWindow()
	--[[elseif command == "load" then
		if (not DueLogDB) or (not DueLogDB.duels[tonumber(args)]) then
			print(true, "Saved duel "..args.." not found")
		else
			if not DueLog.duels then DueLog.duels = {} end
			tinsert(DueLog.duels, DueLogDB.duels[tonumber(args)])
			print(true, "Saved duel "..args.." loaded ("..#DueLog.duels..")")
		end
	elseif command == "loadall" then
		if not DueLogDB then DueLogDB = {duels={}} end
		if not DueLog.duels then DueLog.duels = {} end
		for key, duel in ipairs(DueLogDB.duels) do
			tinsert(DueLog.duels, duel)
			print(true, "Loaded duel "..duel.duel1.." - "..duel.duel2.." (Winner: "..duel.winner..") Duration: "..round(duel.duration,2).." seconds ("..#DueLog.duels..")")
		end
	elseif command == "save" then
		if (not DueLog.duels) or (not DueLog.duels[tonumber(args)]) then print(true, "Duel "..args.." not found") return end
		if not DueLogDB then DueLogDB = {duels={}} end
		tinsert(DueLogDB.duels, DueLog.duels[tonumber(args)]) 
		print(true, "Saved duel "..args)
	elseif command == "delete" then
		if (not DueLog.duels) or (not DueLog.duels[tonumber(args)]) then print(true, "Duel "..args.." not found") return end
		tremove(DueLog.duels, tonumber(args))
		print(true, "Duel "..args.." deleted")
	elseif command == "delsaved" then
		if (not DueLogDB) or (not DueLogDB.duels[tonumber(args)]) then print(true, "Saved duel "..args.." not found") return end
		tremove(DueLogDB.duels, tonumber(args))
		print(true, "Saved duel "..args.." deleted")
	--]]elseif command == "transmit" then
		DueLog:TransmitWindow()
	else
		if command ~= "help" and args ~= "" then
			args = tostring(args)
			print(true, "'"..args.."' is not a valid option for /duelog")
			print(true, "Valid options:")
		else
			print(true, "DueLog help")
			print(true, "List of valid options to /duelog:")
		end
		print(true, "/duelog list - Show list of duels")
		print(true, "/duelog transmit - Opens the duel transmission window")
		print(true, "/duelog help - Display this help")
	end
end
SLASH_DUELOG1 = "/duelog"
--[===[@debug@
SlashCmdList["DUMMY_TEX"] = function(input) local f = CreateFrame("Button", nil, UIParent) f:SetPoint("CENTER") f:SetWidth(50) f:SetHeight(50) f:SetNormalTexture(input) f:SetPushedTexture(input) end
SLASH_DUMMY_TEX1 = "/dummytex"
SlashCmdList["DUMPLOG"] = function(input) if (not DueLog.duels) or (not DueLog.duels[tonumber(input)]) then return end for num, logdata in ipairs(DueLog.duels[tonumber(input)].combatlog) do print(false, num, (logdata.time-DueLog.duels[tonumber(input)].started), unpack(logdata)) end end
SLASH_DUMPLOG = "/dumplog"
--@end-debug@]===]